; (function () {
    'use strict';

    if (window.CloudHubJSBridge) return;
    
    // 回调管理
    var callbacks = {
        // 序号
        index: 0,
        // 集合
        map: {},
        // 注册一个 callback 并返回它的 id（所有id为偶数）
        register: function (callback) {
            this.index += 2;
            var id = '' + this.index;

            // 被回调后从 map 中删除
            if (typeof callback == 'function') {
                this.map[id] = function (message) {
                    // delete this.map[this.id]
                    this.cb.call(null, message)
                }.bind({ map: this.map, id: id, cb: callback});
            };
        
            return id;
        },
        // 根据 id 触发 callback
        invoke: function (id, message) {
            var cb = this.map[id + ''];

            if (typeof cb === 'function') {
                cb(message);
            }
        }
    };

    var events = {};

    // 解析命名空间
    var methodParse = function (method) {
        var obj = method.match(/(.+)\.([^\.]+)/);

        if (obj && obj.length === 3) {
            return {
                name: obj[2],
                ns: obj[1]
            }
        };
        
        return {
            ns: '',
            name: method
        };
    }
    
    // js桥，用于 jssdk 和 native 之间的通信
    var bridge = {
        // js 调用 native 方法
        invoke: function (method, message, callback) {
            var cid = callbacks.register(callback);
            var url;
            if (typeof message === 'undefined') {
                message = '';
            } else {
                message = encodeURIComponent(JSON.stringify(message));
            }
            
            // 解析命名空间
            method = methodParse(method);
            // cloudhub://命名空间:回调号/方法名?参数
            url = "cloudhub://" + method.ns + ":" + cid + "/" + method.name + "?" + message;
            // prompt 触发协议
            window.prompt(url);
            
            return cid;
        },
        // native 回调 js
        // loadUrl("javascript:CloudHubJSBridge.callback(port,resultData)");
        callback: function (callbackid, message) {
            if (typeof message === 'string' && message.match(/^\s*\{/)) {
                try {
                    message = JSON.parse(message);
                } catch (e) {}
            };
            callbacks.invoke(callbackid, message)
        },
        // 绑定事件
        on: function (eventName, callback) {
            if (!events[eventName]) {
                events[eventName] = [];
            };
            events[eventName].push(callback);
        },
        // 触发事件
        trigger: function(eventName, params) {
            (events[eventName] || []).forEach(function (callback) {
                if (typeof callback === 'function') {
                    callback(params);
                }
            });
        }
    };

    // 创建一个全局的桥，供原生调用
    window.CloudHubJSBridge = bridge;

    // 等待 sdk 加载完毕
        var timeout = 3000;
        var interval = 9;
        var timer = setInterval(function () {
//            console.info(timeout)
            if (timeout <= 0) {
                // 等待超时
                clearInterval(timer);
            } else if (typeof window.__onJSBridgeReady === 'function') {
                // 等待成功
                clearInterval(timer);
                bridge.isReady = true;
                window.__onJSBridgeReady(bridge);
                window.__onJSBridgeReady = null;
            } else {
                // 继续等待
                timeout -= interval;
            }
        }, interval);
}());
