; (function (root, factory) {
    // requirejs, seajs
    if (typeof define === 'function' && (define.amd || define.cmd)) {
        define([], factory);
        // commonjs
    } else if (typeof exports === 'object') {
        module.exports = factory();
        // script
    } else {
        root.cloudhub = factory();
    }
}(this, function () {
    'use strict';
    
    var ua = navigator.userAgent,
        // 是否在云之家移动端
        isCloudHubMobile = !!ua.match(/Qing\/.*;(iPhone|iOS|Android).*/);

    var apis = [];
    var events = [
		'backbutton',
		'online',
		'offline',
		'pause',
		'resume',
		'swipeRefresh',
		'appLinkResponse',
		'internalPageLinkResponse',
		'networkEvent',
		'hostTaskEvent',
		'autoCheckIn'
    ];
    var config;
    var errorHandler = function (msg, code) {};
    var sdk = {
        config: function (cfg) {
            if (cfg) {
                config = {
                    appId: cfg.appId || -1,
                    eid: cfg.eid,
                    timeStamp: cfg.timeStamp,
                    nonceStr: cfg.nonceStr,
                    signature: cfg.signature,
                    jsApiList: cfg.jsApiList
                }
            }
            
            return sdk;
        },
        ready: function (callback) {
            if (!config) return alert('请先调用config()方法'), sdk;
            if (typeof callback !== 'function') return alert('请为ready()方法传入回调函数'), sdk;

            var ready = function (bridge) {
                if (!bridge) return alert('JSBridge初始化失败');

                // 全局事件监听
                events.forEach(function(name) {
                    bridge.on(name, function(params) {
                        var event = document.createEvent("HTMLEvents");
                        event.data = event.detail = params;
                        event.initEvent(name);
                        document.dispatchEvent(event);
                    });
                })

                // 发起鉴权
                bridge.invoke('config', config, function (e) {
                    if (!e.success) {
                        errorHandler('权限校验失败', 1001)
                    } else {
                        // 回调
                        callback();
                    };
                })
            }

            // 如果Bridge已经就绪，直接调用ready方法
            // 否则等待Bridge就绪
            if (typeof CloudHubJSBridge === 'undefined' || !CloudHubJSBridge.isReady) {
                window.__onJSBridgeReady = ready;
            } else {
                ready(CloudHubJSBridge);
            }
            
            return sdk;
        },
        error: function (handler) {
            if (typeof handler === 'function') errorHandler = handler;
            return sdk;
        }
    };

    // 客户端检查
    if (!isCloudHubMobile) {
        sdk = {
            config: function () { return sdk; },
            ready: function () { return sdk; },
            error: function (handler) {
                if (typeof handler === 'function') handler('请使用云之家客户端打开本页面');
                return sdk;
            }
        }
    };

    return sdk;
}));