;(function () {
    var info = window.info
    var tools = window.tools
    // 常量定义
    var getCloudLoginInfoUrl = 'https://qing.ik3cloud.com/api/xt/getCloudLoginInfo'
    var kdcloudAppids =  ['10702', '10694', '10498', '10701', '10703', '10803', '10732', '10675', '10697']

    var sessionidCache = {}
    var getSessionID = function (appids, tickets, requestFn) {
        return new Promise(function (resolve, reject) {
            var isArray = Array.isArray(appids)
            var now = new Date().getTime()

            // 把参数格式化成不重复的数组
            if (!isArray) {
                appids = [appids]
            }

            // 数组去重
            var map = {}
            appids = appids.filter(function (n) {
                var has = map[n + '']
                map[n + ''] = true
                return !has
            })

            var localSessionId = []
            var requestAppids = []

            appids.forEach(function (appid) {
                var cache = sessionidCache[appid]
                if (cache && cache.value && cache.expired > now) {
                    localSessionId.push(cache.value)
                } else {
                    requestAppids.push(appid)
                }
            })

            // 如果所有appid都有有效缓存，直接返回
            if (appids.length === localSessionId.length) {
                resolve(localSessionId)
                return
            }

            // 标记状态
            requestAppids.forEach(function (appid) {
                sessionidCache[appid] = {
                    status: 'loading',
                    value: '',
                    expired: 0
                }
            })

            requestFn({
                method: 'post',
                serializer: 'json',
                url: getCloudLoginInfoUrl,
                data: {
                    appTickets: requestAppids.map(function (appId) {
                        return {
                            appId: appId,
                            ticket: tickets[appId]
                        }
                    })
                }
            }).then(function (e) {
                var success = e.success
                var data = e.data
                if (success && data) {
                    // 60秒后过期
                    var expired = new Date().getTime() + 60 * 1000
                    var now = new Date().getTime()
                    
                    data.forEach(function (n) {
                        sessionidCache[n.appId] = {
                            status: 'ready',
                            value: n,
                            expired: expired
                        }
                    })
                    
                    var sessionids = appids.map(function (appid) {
                        var cache = sessionidCache[appid]
                        if (cache && cache.value && cache.expired > now) {
                            return cache.value
                        } else {
                            return {}
                        }
                    })

                    resolve(sessionids)
                } else {
                    // 获取失败了，要从缓存中移除
                    requestAppids.forEach(function (appid) {
                        var sessionid = sessionidCache[appid]
                        if (sessionid && sessionid.status === 'loading') {
                            delete sessionidCache[appid]
                        }
                    })
                    reject(new Error('get sessionids failed'))
                }
            }).catch(reject)
        })
    }

    // 延时批量获取金蝶云sessionid
    var kdcloudSessionWaiter = function (id, ticket, requester, callback, onerror) {
        // appid 加入队列
        if (!kdcloudSessionWaiter.appids.includes(id)) {
            kdcloudSessionWaiter.appids.push(id)
        }

        // 回调函数加入队列
        (kdcloudSessionWaiter.callbacks[id] = kdcloudSessionWaiter.callbacks[id] || []).push(callback)

        kdcloudSessionWaiter.tickets[id] = ticket

        // 重设计时器
        clearTimeout(kdcloudSessionWaiter.timer)
        kdcloudSessionWaiter.timer = setTimeout(function () {
            // 转移
            var appids = kdcloudSessionWaiter.appids
            var tickets = kdcloudSessionWaiter.tickets
            var waitings = kdcloudSessionWaiter.callbacks
            // 清空队列
            kdcloudSessionWaiter.appids = []
            kdcloudSessionWaiter.tickets = {}
            kdcloudSessionWaiter.callbacks = {}
            
            // 开始获取tickets
            getSessionID(appids, tickets, requester).then(function (sessionids) {
                appids.forEach(function (appid, index) {
                    var sessionid = sessionids[index]
                    var callbacks = waitings[appid] || []
                    callbacks.forEach(function (cb) { cb(sessionid, appid) })
                })
            }).catch(function (e) {
                if (typeof onerror === 'function') {
                    onerror(e)
                }
            })
        }, kdcloudSessionWaiter.delay)
    }
    kdcloudSessionWaiter.appids = []
    kdcloudSessionWaiter.tickets = {}
    kdcloudSessionWaiter.callbacks = {}
    kdcloudSessionWaiter.delay = 50

    var devmode = window.localStorage && localStorage.getItem('devmode')

    function debugToast (msg) {
        if (devmode) {
            tools.toast(msg)
        }
    }
    
    function requestHook (opt, appId, ticket, requester) {
        opt.headers = opt.headers || {}
        return new Promise(function (resolve, reject) {
            if (appId && ticket && kdcloudAppids.includes(appId)) {
                // 金蝶云卡片的请求，需附带session在headers
                kdcloudSessionWaiter(appId, ticket, requester, function (sessionid) {
                    var msg

                    if (sessionid.sessionId) {
                        opt.headers['kdservice-sessionid'] = sessionid.sessionId
                        opt.url = tools.pathResolve(sessionid.url, opt.url)
                        resolve(opt)
                    } else {
                        msg = sessionid.message || '获取sessionId失败'
                        debugToast(msg)
                        reject(new Error(msg))
                    }
                }, function () {
                    var msg = '获取sessionId失败'
                    debugToast(msg)
                    reject(new Error(msg))
                })
            } else {
                resolve(opt)
            }
        })
    }

    info._extendConfig({
        cardlist: {
            settings: {
                enable: true
            },
            // 自定义卡片顺序
            order: [
                'starry-sky-amoeba',
                'starry-sky-gross-margin',
                'starry-sky-receivable',
                'starry-sky-orders',
                'starry-sky-gross-business-approval',
                'starry-sky-gross-work-info'
            ]
        },
        requestHook: requestHook,
        // 是否禁用通知卡片
        disableNotice: true
    })
}())
